﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Activities;
using System.IO;
using Benday.TeamBuild2012.Activities;

namespace Benday.TeamBuild2012.UnitTests
{
    [TestClass]
    public class WorkflowFixture : UnitTestBase
    {
        [TestMethod]
        public void WorkflowFixture_CreateAndDeleteIisWebApplication()
        {
            DeleteApplicationIfItExists(STR_DefaultWebSite, "TestApplication");

            // Create an instance of our test workflow
            var workflow = new WorkflowForIIS();

            // Create the workflow run-time environment
            var workflowInvoker = new WorkflowInvoker(workflow);

            // New up an instance of our mock BuildDetail object to add to the Workflow
            // environment's extensions
            workflowInvoker.Extensions.Add(new BuildDetail());

            // Invoke the workflow and capture the outputs
            IDictionary<String, Object> outputs = workflowInvoker.Invoke();            
        }

        [TestMethod]
        public void WorkflowFixture_DeployDatabase()
        {
            // Create an instance of our test workflow
            var workflow = new WorkflowForDeployDatabase();

            // Create the workflow run-time environment
            var workflowInvoker = new WorkflowInvoker(workflow);

            // New up an instance of our mock BuildDetail object to add to the Workflow
            // environment's extensions
            workflowInvoker.Extensions.Add(new BuildDetail());

            // Invoke the workflow and capture the outputs
            IDictionary<String, Object> outputs = workflowInvoker.Invoke();
        }

        [TestMethod]
        public void WorkflowFixture_MoveFile()
        {
            var sourceFile = Path.Combine(TestContext.TestDeploymentDir, "sourceFile.txt");
            var targetFile = Path.Combine(TestContext.TestDeploymentDir, "targetFile.txt");

            // make sure the files don't exist
            File.Delete(sourceFile);
            File.Delete(targetFile);

            // create the files
            File.WriteAllText(sourceFile, "Source file.");
            File.WriteAllText(targetFile, "Original target file.");

            // mark the target as Read Only
            File.SetAttributes(targetFile, FileAttributes.ReadOnly);

            // Create an instance of our test workflow
            var workflow = new WorkflowForMoveFile();

            // Create the workflow run-time environment
            var workflowInvoker = new WorkflowInvoker(workflow);

            // New up an instance of our mock BuildDetail object to add to the Workflow
            // environment's extensions
            BuildDetail buildDetailInstance = new BuildDetail();
            workflowInvoker.Extensions.Add(buildDetailInstance);

            Dictionary<string, object> variables = new Dictionary<string, object>();

            variables.Add("SourcePath", sourceFile);
            variables.Add("TargetPath", targetFile);

            // Invoke the workflow and capture the outputs
            IDictionary<String, Object> outputs = workflowInvoker.Invoke(variables);

            Assert.IsTrue(File.Exists(targetFile), "Target file didn't exist.");
            var targetFileContents = File.ReadAllText(targetFile);

            Assert.AreEqual<string>("Source file.", targetFileContents, "Target file contents was wrong.");
        }

        [TestMethod]
        public void WorkflowFixture_UpdateConnectionString()
        {
            var workflow = new WorkflowForUpdateConnectionString();

            // Create the workflow run-time environment
            var workflowInvoker = new WorkflowInvoker(workflow);

            // New up an instance of our mock BuildDetail object to add to the Workflow
            // environment's extensions
            BuildDetail buildDetailInstance = new BuildDetail();
            workflowInvoker.Extensions.Add(buildDetailInstance);

            Dictionary<string, object> variables = new Dictionary<string, object>();

            string configFile = Path.Combine(TestContext.TestDeploymentDir, "web.config");

            variables.Add("SourcePath", configFile);

            // Invoke the workflow and capture the outputs
            IDictionary<String, Object> outputs = workflowInvoker.Invoke(variables);            

            var value = DatabaseConfigurationFileEditor.GetConnectionString(configFile, "asdf");

            Assert.AreEqual<string>("updated value", value, "Value was wrong.");
        }

        [TestMethod]
        public void WorkflowFixture_UpdateAspSessionStateProvider_AllNullPerformsNoChanges()
        {
            var workflow = new WorkflowForUpdateAspNetSessionState();

            // Create the workflow run-time environment
            var workflowInvoker = new WorkflowInvoker(workflow);

            // New up an instance of our mock BuildDetail object to add to the Workflow
            // environment's extensions
            BuildDetail buildDetailInstance = new BuildDetail();
            workflowInvoker.Extensions.Add(buildDetailInstance);

            Dictionary<string, object> variables = new Dictionary<string, object>();

            string configFile = Path.Combine(TestContext.TestDeploymentDir, "web.config");

            var original = WebConfigEditor.GetSessionStateConfiguration(configFile);
            var originalMode = original.Attribute("mode").Value;

            variables.Add("SourcePath", configFile);
            variables.Add("AspSessionMode", null);

            // Invoke the workflow and capture the outputs
            IDictionary<String, Object> outputs = workflowInvoker.Invoke(variables);

            var updated = WebConfigEditor.GetSessionStateConfiguration(configFile);
            var updatedMode = updated.Attribute("mode").Value;

            Assert.AreEqual<string>(originalMode, updatedMode, "Mode shouldn't change on null value.");
        }

        [TestMethod]
        public void WorkflowFixture_UpdateAspSessionStateProvider_NonNullPerformsChanges()
        {
            var workflow = new WorkflowForUpdateAspNetSessionState();

            // Create the workflow run-time environment
            var workflowInvoker = new WorkflowInvoker(workflow);

            // New up an instance of our mock BuildDetail object to add to the Workflow
            // environment's extensions
            BuildDetail buildDetailInstance = new BuildDetail();
            workflowInvoker.Extensions.Add(buildDetailInstance);

            Dictionary<string, object> variables = new Dictionary<string, object>();

            string configFile = Path.Combine(TestContext.TestDeploymentDir, "web.config");

            variables.Add("SourcePath", configFile);

            var expected = Guid.NewGuid().ToString();

            variables.Add("AspSessionMode", expected);

            // Invoke the workflow and capture the outputs
            IDictionary<String, Object> outputs = workflowInvoker.Invoke(variables);

            var updated = WebConfigEditor.GetSessionStateConfiguration(configFile);
            var updatedMode = updated.Attribute("mode").Value;

            Assert.AreEqual<string>(expected, updatedMode, "Mode should have changed.");
        }
    }
}
